code segment public 'code'
                assume  cs:code, ds:code, ss:code

                public  PrintString, PrintCrLf, PrintChar, KeyPressed
                public  ReadKey, FlushBuffer, InitSerial

;==============================================================================
;
; IO_Device:    0    = CONSOLE
;               3f8h = SERIAL PORT 1
;               2f8h = SERIAL PORT 2
;               3e8h = SERIAL PORT 3
;               2e8h = SERIAL PORT 4
;

IO_Device       equ     2f8h

;==============================================================================
;
; Not device specific routines
;

PrintString:    lodsb
                or      al,al
                je      EndOfString
                call    PrintChar
                jmp     PrintString
EndOfString:    ret

PrintCrLf:      mov     al,13
                call    PrintChar
                mov     al,10
                jmp     PrintChar

FlushBuffer:    call    KeyPressed
                jz      BufferEmpty
                call    ReadKey
                jmp     FlushBuffer
BufferEmpty:    ret

;==============================================================================
;
; Specialized for CONSOLE input/output
;

if IO_Device eq 0

PrintChar:      push    bx
                mov     ah,0eh
                mov     bx,7
                int     10h
                pop     bx
                ret

KeyPressed:     mov     ah,1
                int     16h
                ret

ReadKey:        mov     ah,0
                int     16h
                cmp     al,'a'
                jb      UpperCase
                cmp     al,'z'
                ja      UpperCase
                sub     al,20h
UpperCase:      ret

InitSerial:     ret

;==============================================================================
;
; Specialized for serial input/output
;

else

KeyBufSize      equ     20h

KeyBuffer       db      KeyBufSize dup(?)
KeyInPtr        dw      0
KeyOutPtr       dw      0
Pause           db      0

PrintChar:      cmp     al,19
                je      DontPrint
                push    dx
                push    ax
                mov     dx,IO_Device + 5
WaitReady:      in      al,dx
                test    al,20h          ; Transmitter empty
                jz      WaitReady
WaitPause:      call    KeyPressed
                cmp     cs:Pause,0
                jne     WaitPause
                pop     ax
                mov     dx,IO_Device
                out     dx,al
                pop     dx
DontPrint:      ret

KeyPressed:     push    dx
                mov     dx,IO_Device + 5
                in      al,dx
                test    al,1
                jz      NoData
                mov     dx,IO_Device
                in      al,dx
                cmp     al,19
                jne     NotXOFF
                mov     cs:Pause,-1
                jmp     short NoData
NotXOFF:        cmp     al,17
                jne     NotXON
                mov     cs:Pause,0
                jmp     short NoData
NotXON:         push    bx
                mov     bx,cs:KeyInPtr
                and     bx,KeyBufSize - 1
                mov     cs:KeyBuffer[bx],al
                inc     cs:KeyInPtr
                pop     bx
NoData:         pop     dx
                mov     ax,cs:KeyOutPtr
                cmp     ax,cs:KeyInPtr
                ret

ReadKey:        call    KeyPressed
                jz      ReadKey
                push    bx
                mov     bx,cs:KeyOutPtr
                and     bx,KeyBufSize - 1
                mov     al,cs:KeyBuffer[bx]
                inc     cs:KeyOutPtr
                pop     bx
                cmp     al,'a'
                jb      UpperCase
                cmp     al,'z'
                ja      UpperCase
                sub     al,20h
UpperCase:      ret

InitSerial:     mov     dx,IO_Device + 3        ; Init COM2:
                mov     al,83h
                out     dx,al
                mov     dx,IO_Device
                mov     al,6
                out     dx,al
                mov     dx,IO_Device + 1
                mov     al,0
                out     dx,al
                mov     dx,IO_Device + 3
                mov     al,3
                out     dx,al
                mov     dx,IO_Device + 1
                mov     al,0
                out     dx,al
                mov     dx,IO_Device + 2
                mov     al,0c0h
                out     dx,al
                jmp     PrintCrLf

endif

code ends

end
